import requests
import random

HTTP_DEFAULT_TIMEOUT = 20
BROWSER_USER_AGENT = [
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.67 Safari/537.36",
    "Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.125 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1944.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.3319.102 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.2309.372 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.2117.157 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1866.237 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.137 Safari/4E423F",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.116 Safari/537.36 Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.517 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1667.0 Safari/537.36",
]


class HtmlFetcher(object):
    def __init__(self):
        self.http_timeout = HTTP_DEFAULT_TIMEOUT
        # set header
        self.headers = {'User-agent': random.choice(BROWSER_USER_AGENT)}

    def get_html(self, url):
        # do request
        try:
            req = requests.get(url, headers=self.headers, timeout=self.http_timeout)
            if req.encoding == 'ISO-8859-1':
                if req.apparent_encoding is not None:
                    req.encoding = requests.utils.get_encodings_from_content(req.text)[0]
                else:
                    req.encoding = req.apparent_encoding
            elif len(requests.utils.get_encodings_from_content(req.text)) > 0:
                if requests.utils.get_encodings_from_content(req.text)[0] == "GBK":
                    req.encoding = requests.utils.get_encodings_from_content(req.text)[0]
            req.keep_alive = False
            html = req.text
            # print(html)

        except Exception as e:
            print(e)
            html = None
        # print(html)
        # print(req.headers['content-type'])
        # print("response内容的encoding编码:", req.encoding)
        # print("response headers里设置的apparent_encoding编码:", req.apparent_encoding)
        # print("response返回的html header标签里设置的编码:", requests.utils.get_encodings_from_content(req.text))
        return html


class FileFetcher(object):
    @staticmethod
    def get_html(url):
        # do read
        print(url)
        try:
            with open(url, "r", encoding="utf-8") as file_obj:
                html = file_obj.read().strip()
        except Exception as e:
            print(e)
            html = None
        return html



from lxml import etree


class Parser(object):
    @staticmethod
    def raw_to_document(raw_html):
        return etree.HTML(raw_html)

    @staticmethod
    def xpathSelect(selector, xpath_lan):
        return selector.xpath(xpath_lan)



"""
Extract the content of the web page
"""

from .crawler import Crawler
from .version import __version__


class Stallion(object):
    """
    Main class
    """

    def __init__(self, enable_urls=False):
        self.enable_urls = enable_urls

    def extract(self, url, is_file=False):
        """
        Main method to extract an article object from a URL,
        pass in a url and get back a Article
        """
        cr = Crawler(is_file, self.enable_urls)
        try:
            article = cr.crawl(url)
        except Exception as e:
            print(e)
            return cr.article
        return article

    def shutdown_network(self):
        pass


s = Stallion()
extract = s.extract



"""
Extractor
"""
from urllib.parse import urljoin


class BaseExtractor(object):
    def __init__(self, parser, article):
        # parser
        self.parser = parser

        # article
        self.article = article


class TitleExtractor(BaseExtractor):
    def get_title(self):
        """
        Fetch the article title and analyze it
        """
        command = '//title/text()'
        content = self.parser.xpathSelect(self.article.doc, command)
        if content is not None and len(content) > 0:
            return content[0].strip()
        return ''

    def extract(self):
        return self.get_title()


class MetasExtractor(BaseExtractor):
    def get_meta_content(self, meta_name):
        """
        Extract a given meta content form document
        """
        command = '//meta[translate(@name,"ABCDEFGHJIKLMNOPQRSTUVWXYZ",abcdefghjiklmnopqrstuvwxyz)="{0}"]/@content'.format(
            meta_name)
        content = self.parser.xpathSelect(self.article.doc, command)
        if content is not None and len(content) > 0:
            return content[0].strip()
        return ''

    def get_meta_description(self):
        """
        if the article has meta description set in the source, use that
        """
        return self.get_meta_content("description")

    def get_meta_keywords(self):
        """
        if the article has meta words set in the source, use that
        """
        return self.get_meta_content("keywords")

    def extract(self):
        return {
            "description": self.get_meta_description(),
            "keywords": self.get_meta_keywords(),
        }


class H1Extractor(BaseExtractor):
    def get_h1(self):
        """
        Fetch the article title and analyze it
        """
        command = '//h1/text()'
        content = self.parser.xpathSelect(self.article.doc, command)
        if content is not None and len(content) > 0:
            return content[0].strip()
        return ''

    def extract(self):
        return self.get_h1()


class ContentExtractor(BaseExtractor):
    def get_content(self):
        """
        Fetch the article title and analyze it
        """
        command = 'string()'
        content = self.parser.xpathSelect(self.article.doc, command)
        if content is not None and len(content) > 0:
            return content.strip()
        return ''

    def extract(self):
        return self.get_content()


class UrlListExtractor(BaseExtractor):
    def filer_url(self, url):
        """
        join abnormal url
        """
        url = url.strip()
        if not url.startswith("http"):
            return urljoin(self.article.url_domain, url)
        return url

    def clean_url(self, url_list):
        """
        Clear non-url,Abnormal url add domain,delete duplicate
        """
        url_out = [self.filer_url(url) for url in url_list]
        return list(set(url_out))

    def get_urls(self):
        """
        Fetch the article title and analyze it
        """
        command = '//a/@href'
        url_list = self.parser.xpathSelect(self.article.doc, command)
        if url_list is not None and len(url_list) > 0:
            return self.clean_url(url_list)
        return []

    def extract(self):
        return self.get_urls()



# -*- coding: utf-8 -*-


from setuptools import setup, find_packages
from stallions import __version__

VERSION = __version__

setup(
    name='stallions',
    version=VERSION,
    description='Extract the content of the web page.',
    license='',
    author='galen',
    author_email='galen.wang@rtbasia.com',
    classifiers=[
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    url='https://git.rtbasia.com/galen/python_stallion',
    keywords='Web spider',
    packages=find_packages(),
    install_requires=[
        'lxml',
        'requests',
    ],
)



import re


class OutputFormatter(object):
    @staticmethod
    def clean_content(content):
        """streamline \r\n\ space"""
        # content = re.sub(r'[^\u4e00-\u9fa5]+', ' ', content) # Eliminate Chinese characters
        return re.sub('[\r\n\t ]+', ' ', content).replace('\xa0', '').strip()


class StandardOutputFormatter(OutputFormatter):
    pass


class EliminateScript:
    @staticmethod
    def delete_all_tag(html_raw):
        # <!--done-->  style script
        html_raw = EliminateScript.delete_notes(html_raw)
        html_raw = EliminateScript.delete_tags(html_raw, "style")
        return EliminateScript.delete_tags(html_raw, "script")

    @staticmethod
    def delete_tags(html_raw, tags):
        html_list = html_raw.split("<{0}".format(tags))
        fresh_html = ""
        if html_raw.startswith("<{0}".format(tags)):
            for i, htm in enumerate(html_list):
                if "</{0}>".format(tags) not in htm:
                    continue
                fresh_html += htm.split("</{0}>".format(tags))[-1]
        else:
            for i, htm in enumerate(html_list):
                if i == 0:
                    fresh_html += htm
                    continue
                if "</{0}>".format(tags) not in htm:
                    continue
                fresh_html += htm.split("</{0}>".format(tags))[-1]
        # for htm in html_list:
        #     if "</{0}>".format(tags) not in htm:
        #         fresh_html += htm
        #         continue
        #     fresh_html += htm.split("</{0}>".format(tags))[-1]
        return fresh_html

    @staticmethod
    def delete_notes(html_raw):
        # delete note
        html_list = html_raw.split("<!--")
        fresh_html = ""
        if html_raw.startswith("<!--"):
            for i, htm in enumerate(html_list):
                if "-->" not in htm:
                    continue
                fresh_html += htm.split("-->")[-1]
        else:
            for i, htm in enumerate(html_list):
                if i == 0:
                    fresh_html += htm
                    continue
                if "-->" not in htm:
                    continue
                fresh_html += htm.split("-->")[-1]
        return fresh_html



class Article(object):
    def __init__(self):
        self.url_domain = ""
        self.meta_description = ""
        self.meta_keywords = ""
        self.title = ""
        self.h1 = ""
        self.content = ""
        self.raw_html = ""
        self.url_list = []

        # the lxml Document object
        self.doc = None

    @property
    def infos(self):
        data = {
            "meta_description": self.meta_description,
            "meta_keywords": self.meta_keywords,
            "title": self.title,
            "h1": self.h1,
            "content": self.content,
            "url_list": self.url_list
        }
        return data



from .article import Article
from .extractors import TitleExtractor, H1Extractor, ContentExtractor, MetasExtractor, UrlListExtractor
from .outputformat import OutputFormatter, EliminateScript

from .network import HtmlFetcher, FileFetcher
from .parsers import Parser

get_parser = Parser()


class Crawler(object):
    def __init__(self, is_file, enable_urls=False):
        # config
        self.enable_urls = enable_urls

        # parser
        self.parser = get_parser

        # article
        self.article = Article()

        # metas extractor
        self.metas_extractor = self.get_metas_extractor()

        # title extractor
        self.title_extractor = self.get_title_extractor()

        # h1 extractor
        self.h1_extractor = self.get_h1_extractor()

        # content extractor
        self.content_extractor = self.get_content_extractor()

        # url_list extractor
        self.url_list_extractor = self.get_url_list_extractor()
        # html fetcher
        if is_file:
            self.html_fetcher = FileFetcher()
        else:
            self.html_fetcher = HtmlFetcher()

    def crawl(self, url):
        raw_html = self.html_fetcher.get_html(url)
        if raw_html is None:
            return self.article
        raw_html = EliminateScript().delete_all_tag(raw_html)
        self.article.url_domain = url
        # filter js script
        # raw_html = cleaner.clean_html(raw_html)
        doc = self.parser.raw_to_document(raw_html)
        self.article.raw_html = raw_html
        self.article.doc = doc
        metas = self.metas_extractor.extract()
        self.article.meta_description = OutputFormatter.clean_content(metas['description'])
        self.article.meta_keywords = OutputFormatter.clean_content(metas['keywords'])
        self.article.title = OutputFormatter.clean_content(self.title_extractor.extract())
        self.article.h1 = OutputFormatter.clean_content(self.h1_extractor.extract())
        self.article.content = OutputFormatter.clean_content(self.content_extractor.extract())
        if self.enable_urls:
            self.article.url_list = self.url_list_extractor.extract()
        return self.article

    def get_metas_extractor(self):
        return MetasExtractor(self.parser, self.article)

    def get_title_extractor(self):
        return TitleExtractor(self.parser, self.article)

    def get_h1_extractor(self):
        return H1Extractor(self.parser, self.article)

    def get_content_extractor(self):
        return ContentExtractor(self.parser, self.article)

    def get_url_list_extractor(self):
        return UrlListExtractor(self.parser, self.article)
